/**
 * LICENSE 
 * Bonita
 * Copyright (C) 2005 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.proed.chi.test;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.ow2.proed.Trace;
import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.BlockActivity;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.RouteActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.Transition;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.ProcessActivity.ExecutionMode;
import org.ow2.proed.graphModel.exception.TransitionException;
import org.ow2.proed.resources.PropertiesManager;

/**
 * CollectionElementTest.java Created on 21 juin 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: CollectionElementTest.java,v 1.3 2005/08/04 15:18:46 gonthiem
 *          Exp $
 *          <p>
 */
public class CollectionElementTest extends TestCase {

  private ProEdProject project;

  // trace for error
  private Trace trace;

  // process activities to test
  private AtomicActivity a1;

  private BlockActivity a2;

  private RouteActivity a3;

  private Subflow s;

  // transitions to test
  private Transition t1;

  private Transition t2;

  private Transition t3;

  private Transition t4;

  private final PropertiesManager propManager = new PropertiesManager();

  public CollectionElementTest(final String name) {
    super(name);
  }

  @Override
  protected void setUp() {

    this.project = new ProEdProject("Project1", false);
    final WorkflowProcesses workflows = new WorkflowProcesses();
    final WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
    this.project.setMyProcesses(workflows);
    workflows.add(mainWorkflow);

    // set up all process activities
    this.a1 = new AtomicActivity("Activity 1", null, null, null, ExecutionMode.Synchronous, mainWorkflow, null, null);
    this.a2 = new BlockActivity("Activity 2", null, null, null, ExecutionMode.Synchronous, this.project.getMyProcesses().getMainWorkflow(), null);
    this.a3 = new RouteActivity("Activity 3", null, null, null, ExecutionMode.Synchronous, mainWorkflow, null);
    this.s = new Subflow("subflow 1", null, null, ExecutionMode.Synchronous, mainWorkflow, null);

    try {
      // set up good transitions
      this.t1 = new Transition(this.a1, this.a2, mainWorkflow);
      Assert.assertNotNull(this.t1);
      this.t2 = new Transition(this.a2, this.a3, mainWorkflow);
      Assert.assertNotNull(this.t2);
      this.t3 = new Transition(this.a1, this.s, mainWorkflow);
      Assert.assertNotNull(this.t3);
    } catch (final TransitionException e2) {
      System.out.println(e2.getMessage());
    }
    this.trace = new Trace(true, true);

    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.a1);
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.a2);
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.a3);
    this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.s);

    Assert.assertNotNull(this.project.getMyProcesses().getMainWorkflow().getProcessActivities());
    Assert.assertEquals(this.project.getMyProcesses().getMainWorkflow().getProcessActivities().size(), 4);

    this.project.getMyProcesses().getMainWorkflow().addTransitions(this.t1.getSource(), this.t1.getTarget());
    this.project.getMyProcesses().getMainWorkflow().addTransitions(this.t2.getSource(), this.t2.getTarget());
    this.project.getMyProcesses().getMainWorkflow().addTransitions(this.t3.getSource(), this.t3.getTarget());

    Assert.assertNotNull(this.project.getMyProcesses().getMainWorkflow().getTransitions());
    Assert.assertEquals(this.project.getMyProcesses().getMainWorkflow().getTransitions().size(), 3);

  }

  public void addElementinWorkflow(final WorkflowProcess workflow) {
    try {

      // set up good transitions
      this.t4 = new Transition(this.a3, this.s, workflow);
      Assert.assertNotNull(this.t4);

    } catch (final TransitionException e2) {
      System.out.println(e2.getMessage());
    }
    this.project.getMyProcesses().getMainWorkflow().addTransitions(this.t4.getSource(), this.t4.getTarget());

  }

  public void modifyT3(final WorkflowProcess workflow) {
    try {
      this.addElementinWorkflow(workflow);
      this.project.getMyProcesses().getMainWorkflow().getTransition(this.t3).setSource(this.s);
      this.project.getMyProcesses().getMainWorkflow().getTransition(this.t3).setTarget(this.a1);

    } catch (final TransitionException e2) {
      System.out.println(e2.getMessage());
    }
  }

  public void modifyT3Bis(final WorkflowProcess workflow) {
    try {
      this.addElementinWorkflow(workflow);
      this.project.getMyProcesses().getMainWorkflow().getTransition(this.t3).setSource(this.s);
      this.project.getMyProcesses().getMainWorkflow().getTransition(this.t3).setTarget(this.a2);

    } catch (final TransitionException e2) {
      System.out.println(e2.getMessage());
    }
  }

  public void testGetNextElement() {
    System.out.println("\n\n******testGetNextElementWithT4********");
    this.printTrajet();
    System.out.println("**************");
    // get all activities after a1
    final List allActivities = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a1);
    System.out.println("Activities after A1");
    for (final Iterator iter = allActivities.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a2
    final List allActivitiesAfterA2 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a2);
    System.out.println("Activities after A2");
    for (final Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a3
    final List allActivitiesAfterA3 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a3);
    System.out.println("Activities after A3");
    for (final Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }
  }

  public void testGetNextElementWithT4(final WorkflowProcess workflow) {
    this.addElementinWorkflow(workflow);
    System.out.println("\n\n******testGetNextElementWithT4********");
    this.printTrajet();
    System.out.println("**************");
    // get all activities after a1
    final List allActivities = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a1);
    System.out.println("Activities after A1");
    for (final Iterator iter = allActivities.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a2
    final List allActivitiesAfterA2 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a2);
    System.out.println("Activities after A2");
    for (final Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a3
    final List allActivitiesAfterA3 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a3);
    System.out.println("Activities after A3");
    for (final Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after s
    final List allActivitiesAfterS = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.s);
    System.out.println("Activities after S");
    for (final Iterator iter = allActivitiesAfterS.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }
  }

  private void printTrajet() {
    final Set allTransitions = this.project.getMyProcesses().getMainWorkflow().getTransitions();
    for (final Iterator iter = allTransitions.iterator(); iter.hasNext();) {
      final Transition t = (Transition) iter.next();
      System.out.println(t.getSource() + "->" + t.getTarget());
    }
  }

  public void testGetNextElementWithT3Modified(final WorkflowProcess workflow) {
    // workflow like a1->a2->a3->s1->a1
    this.modifyT3(workflow);
    System.out.println("\n\n**************");
    this.printTrajet();
    System.out.println("**************");
    // get all activities after a1
    final List allActivities = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a1);
    System.out.println("Activities after A1");
    for (final Iterator iter = allActivities.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a2
    final List allActivitiesAfterA2 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a2);
    System.out.println("Activities after A2");
    for (final Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a3
    final List allActivitiesAfterA3 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a3);
    System.out.println("Activities after A3");
    for (final Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after s
    final List allActivitiesAfterS = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.s);
    System.out.println("Activities after S");
    for (final Iterator iter = allActivitiesAfterS.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }
  }

  public void testGetNextElementWithT3ModifiedBis(final WorkflowProcess workflow) {
    // workflow like a1->a2->a3->s1->a2
    this.modifyT3Bis(workflow);
    System.out.println("\n\n******testGetNextElementWithT3ModifiedBis********");
    this.printTrajet();
    System.out.println("**************");
    // get all activities after a1
    final List allActivities = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a1);
    System.out.println("Activities after A1");
    for (final Iterator iter = allActivities.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a2
    final List allActivitiesAfterA2 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a2);
    System.out.println("Activities after A2");
    for (final Iterator iter = allActivitiesAfterA2.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after a3
    final List allActivitiesAfterA3 = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.a3);
    System.out.println("Activities after A3");
    for (final Iterator iter = allActivitiesAfterA3.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }

    // get all activities after s
    final List allActivitiesAfterS = this.project.getMyProcesses().getMainWorkflow().getNextActivities(this.s);
    System.out.println("Activities after S");
    for (final Iterator iter = allActivitiesAfterS.iterator(); iter.hasNext();) {
      System.out.println(iter.next());
    }
  }

  public static Test suite() {
    return new TestSuite(CollectionElementTest.class);
  }

}