/**
 * LICENSE 
 * Bonita
 * Copyright (C) 2005 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.proed.chi.test;

import java.io.IOException;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.jdom.Document;
import org.ow2.proed.Trace;
import org.ow2.proed.Utils;
import org.ow2.proed.chi.mainFrame.MainFrame;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.ProcessActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.ProcessActivity.ExecutionMode;
import org.ow2.proed.xpdl.XPDLProEdProject;

/**
 * ImportSubflowTest.java Created on 22 juin 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: ImportSubflowTest.java 4461 2009-03-05 13:22:00Z vigneras $
 *          <p>
 */
public class ImportSubflowTest extends TestCase {

  // trace for error
  private Trace trace;

  private final String fileName = "TestJUnit.xpdl";

  public final static String SUFFIX_ID_CHANGED = "_IMPORT";

  private ProEdProject project;

  WorkflowProcess mainWorkflow;

  private Subflow subflow;

  @Override
  protected void setUp() {
    this.trace = new Trace(true, true);

    this.project = new ProEdProject("Project1", false);
    final WorkflowProcesses workflows = new WorkflowProcesses();
    this.mainWorkflow = new WorkflowProcess("project1");
    this.project.setMyProcesses(workflows);
    workflows.add(this.mainWorkflow);
  }

  public void testImportSubflow() {
    final String[] args = new String[0];
    final MainFrame owner = new MainFrame(true, true, args);
    Document doc = null;
    try {
      doc = Utils.loadXMLDocument(this.fileName);
    } catch (final IOException e) {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    Assert.assertNotNull(doc);

    if (doc != null) {
      // Import this file in a temporary project
      final ProEdProject projectImported = XPDLProEdProject.toModel(doc, owner);
      Assert.assertNotNull(projectImported);

      this.subflow = new Subflow(projectImported.getName(), projectImported.getMyProcesses().getMainWorkflow().getDescription(), ProcessActivity.TYPE_AND_JOIN,
          ExecutionMode.Synchronous, this.mainWorkflow, null);
      this.subflow.setWorkflow(projectImported.getMyProcesses().getMainWorkflow());
      Assert.assertNotNull(this.subflow);

      // add all workflow processes in this process
      this.project.getMyProcesses().addAll(projectImported.getMyProcesses());
      Assert.assertEquals(this.project.getMyProcesses().size() - 1, projectImported.getMyProcesses().size());

      // add this subflow in main workflow
      this.project.getMyProcesses().getMainWorkflow().addProcessActivity(this.subflow);
      Assert.assertEquals(this.project.getMyProcesses().getMainWorkflow().getProcessActivities().size(), 1);

    }

  }

  public static Test suite() {
    return new TestSuite(ImportSubflowTest.class);
  }
}