/**
 * LICENSE 
 * Bonita
 * Copyright (C) 2005 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.proed.chi.test.graphModel;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.ow2.proed.graphModel.AtomicActivity;
import org.ow2.proed.graphModel.Datafield;
import org.ow2.proed.graphModel.DatafieldType;
import org.ow2.proed.graphModel.PerformerAssignment;
import org.ow2.proed.graphModel.ProEdProject;
import org.ow2.proed.graphModel.ProcessActivity;
import org.ow2.proed.graphModel.Subflow;
import org.ow2.proed.graphModel.WorkflowProcess;
import org.ow2.proed.graphModel.WorkflowProcesses;
import org.ow2.proed.graphModel.ProcessActivity.ExecutionMode;

/**
 * AtomicActivityTest.java Created on 11 juil. 2005
 * 
 * @author Ducreux Evelyne - Gonthier Manu�la
 * @version $Id: AtomicActivityTest.java 4407 2009-02-18 16:55:11Z vigneras $
 *          <p>
 */
public class AtomicActivityTest extends TestCase {

  private ProEdProject project;

  private AtomicActivity a1;

  private AtomicActivity a2;

  private AtomicActivity a3;

  private AtomicActivity a4;

  private AtomicActivity a5;

  private PerformerAssignment performerCallback;

  private PerformerAssignment performerProperty;

  private Datafield datafield;

  private ProcessActivity processActivity;

  /*
   * @see TestCase#setUp()
   */
  @Override
  protected void setUp() throws Exception {
    this.project = new ProEdProject("Project1", false);
    final WorkflowProcesses workflows = new WorkflowProcesses();
    final WorkflowProcess mainWorkflow = new WorkflowProcess("project1");
    this.project.setMyProcesses(workflows);
    workflows.add(mainWorkflow);

    this.a1 = new AtomicActivity("Activity 1", null, null, null, ExecutionMode.Synchronous, mainWorkflow, null, null);
    Assert.assertNotNull(this.a1);
    this.a2 = new AtomicActivity("Activity 2", "2nd activity", ProcessActivity.MANUAL_MODE, null, ExecutionMode.Synchronous, mainWorkflow, null, null);
    Assert.assertNotNull(this.a2);
    this.a3 = new AtomicActivity("Activity 3", "super", null, null, ExecutionMode.Synchronous, mainWorkflow, null, null);
    Assert.assertNotNull(this.a3);
    this.a4 = new AtomicActivity("Activity 4", "activity 4", ProcessActivity.AUTO_MODE, ProcessActivity.TYPE_AND_JOIN, ExecutionMode.Synchronous, mainWorkflow,
        null, null);
    Assert.assertNotNull(this.a4);
    Assert.assertNull(this.a5);

    this.processActivity = new Subflow("subflow", "activity to copy", ProcessActivity.TYPE_AND_JOIN, ExecutionMode.Synchronous, mainWorkflow, null);

    this.performerCallback = new PerformerAssignment(PerformerAssignment.CST_CUSTOM, "hero.hook.callback");
    this.performerProperty = new PerformerAssignment(PerformerAssignment.CST_VARIABLE, "datafield");

    this.datafield = new Datafield("datafield", "d1", DatafieldType.STRING, mainWorkflow);
  }

  public void testConstructAtomicActivity() {
    // test construction of atomic activity with processActivity
    this.a5 = AtomicActivity.constructAtomicActivity(this.processActivity);
    System.out.println("\nA5:" + this.a5.printtoString());
    System.out.println("\nProcessActivity:" + this.processActivity.printtoString());

  }

  public void testRemovePerformerAssignment() {
    this.a1.setPerformerAssignment(this.performerCallback);
    this.a1.removePerformerAssignment(this.datafield);
    Assert.assertNotNull(this.a1.getPerformerAssignment());
    this.a1.setPerformerAssignment(this.performerProperty);
    this.a1.removePerformerAssignment(this.datafield);
    Assert.assertNull(this.a1.getPerformerAssignment());

  }

  public static Test suite() {
    return new TestSuite(AtomicActivityTest.class);
  }
}